/*! \file   argc-argv-test.c
    \brief  A demo for argc/argv parameter passing.
    \author Markus L. Noga <noga@inrialpes.fr>
    
    Displays string array passed as argument to program.
    Press run to stop.
*/
    
/*
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is legOS code, released March 19, 1999.
 *
 *  The Initial Developer of the Original Code is Markus L. Noga.
 *  Portions created by Markus L. Noga are Copyright (C) 1999
 *  Markus L. Noga. All Rights Reserved.
 *
 *  Contributor(s): Markus L. Noga <noga@inrialpes.fr>
 */

#include "conio.h"
#include "unistd.h"
#include "direct-button.h"
#include "direct-sensor.h"
#include "direct-ir.h"
#include "sys/tm.h"
#include "rom/lcd.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global variables
//
///////////////////////////////////////////////////////////////////////////////

pid_t t1,t2;

///////////////////////////////////////////////////////////////////////////////
//
// Functions
//
///////////////////////////////////////////////////////////////////////////////

// button event wakeup functions
//
static wakeup_t button_press_wakeup(wakeup_t data) {
  return PRESSED(button_state(),data);
}

static wakeup_t button_release_wakeup(wakeup_t data) {
  return RELEASED(button_state(),data);
}


// show arguments
//
static int show_task(int argc,char **argv) {
  int i;
  
  while(1)
    for(i=0; i<argc; i++) {
      cputs(argv[i]);
      lcd_refresh();
      delay(500);
    }
    
  return 0;
}

// run button -> stop everything.
//
static int stop_task(int argc, char **argv) {
  wait_event(&button_release_wakeup,BUTTON_RUN);	// debounce
  delay(100);

  wait_event(&button_press_wakeup,BUTTON_RUN);	// wait for RUN button
  kill(t1);					// kill driver
  kill(t2);
  
  return 0;					// kernel stops motors
}

// run the tasks
//            
int main(void) {
  char *argv[]={"Hello","world","howw ","are  ","you  ","doing","?    "};
  int argc=7;
  
  
  t1=execi(&show_task,argc,argv,1,DEFAULT_STACK_SIZE);
  t2=execi(&stop_task,0   ,NULL,2,DEFAULT_STACK_SIZE);
 
  tm_start();

  return 0;
}	
