/*! \file   include/string.h
    \brief  string functions
    \author Markus L. Noga <noga@inrialpes.fr>
*/
    
/*
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is legOS code, released March 19, 1999.
 *
 *  The Initial Developer of the Original Code is Markus L. Noga.
 *  Portions created by Markus L. Noga are Copyright (C) 1999
 *  Markus L. Noga. All Rights Reserved.
 *
 *  Contributor(s): Markus L. Noga <noga@inrialpes.fr>
 */

#ifndef __string_h__
#define __string_h__

#include "mem.h"


///////////////////////////////////////////////////////////////////////////////
//
// Functions
//
///////////////////////////////////////////////////////////////////////////////

//!  copy memory block from src to dest.
/*! \param dest destination
    \param src  source
    \param size number of bytes to copy
    
    \warning behaviour is undefined in case source and destination blocks 
             overlap.
*/
extern void memcpy(void* dest,void* src,size_t size);

//! fill memory block with a byte value.
/*! \param s start
    \param c byte fill value
    \param n number of bytes to fill
*/
extern inline void *memset(void* s,int c,size_t n) {
	void *res;
__asm__ __volatile__(
	"  add.w %1,%0\n"
	"0:cmp.w %1,%0\n"
	"  beq 1f\n"
	"  mov.b %2l,@-%0\n"
	"  bra 0b\n"
	"1:"
	: "=r" (res)				// output
	: "r" (s), "r" (c), "0" (n)		// input
	: "cc","memory"				// clobbered (final)
	);
	return res;					
}

//! set memory block [start,end[ to zero.
/*! \param start start
    \param end   end (non-inclusive).
    \bug FIXME: seems buggy. memset() usage advised.
*/    
extern inline void mem_clear(void* start,void* end) {
	register void* _start __asm__ ("r0")=start;
	register void* _end   __asm__ ("r1")=end;
__asm__ __volatile__(
	"jsr @0x0436:0"
	: 					// output
	:					// input
	: "r0","r1","r2","cc","memory"		// clobbered (final)
	);					
}

//! Copy null-terminated string from src to dest
/*! \param  src source
    \param  dest destination
    \return pointer to dest
*/    
extern char* strcpy(char *dest,const char *src);

//! Determine string length
/*! \param  s string
    \param  s2 second string
    \return string length
*/    
extern int strlen(const char *s);

//! Compare two strings
/*! \param  s1 first string
    \param  s2 second string
    \return <0: s1<s2, ==0: s1==s2, >0: s1>s2
*/    
extern int strcmp(const char *s1,const char *s2);

#endif
