/*! \file   include/unistd.h
    \brief  reduced UNIX standard library
    \author Markus L. Noga <noga@inrialpes.fr>
*/
    
/*
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is legOS code, released March 19, 1999.
 *
 *  The Initial Developer of the Original Code is Markus L. Noga.
 *  Portions created by Markus L. Noga are Copyright (C) 1999
 *  Markus L. Noga. All Rights Reserved.
 *
 *  Contributor(s): Markus L. Noga <noga@inrialpes.fr>
 */

#ifndef __unistd_h__
#define __unistd_h__

#include "tm.h"
#include "time.h"

///////////////////////////////////////////////////////////////////////////////
//
// Functions
//
///////////////////////////////////////////////////////////////////////////////

//
// execute an image
// args: start address of code to execute
//       stack size for new process.
// retval: -1: fail, else pid
//
extern pid_t execi(int (*code_start)(int,char**),int argc, char **argv,
                   priority_t priority,size_t stack_size);


//
// exit task, returning code.
//
extern void exit(int code) __attribute__ ((noreturn));


//
// yield the rest of the current timeslice
//
extern void yield(void);


//
// suspend process until wakeup function is non-null
// wakeup function is called in task scheduler context
// retval: wakeup() return value
//
extern wakeup_t wait_event(wakeup_t (*wakeup)(wakeup_t),wakeup_t data);


//
// sleep wakeup function
// will wake up if data <= system timer
//
extern wakeup_t tm_sleep_wakeup(wakeup_t data);


//! delay execution allowing other tasks to run.
/*! \param sec sleep duration in seconds
    \return number of seconds left if interrupted, else 0.
    \bug interruptions not implemented.
*/
extern inline unsigned int sleep(unsigned int sec) {
  (void) wait_event(&tm_sleep_wakeup,sys_time+SECS_TO_TICKS(sec));
  return 0;
}


//! delay execution allowing other tasks to run.
/*! \param msec sleep duration in milliseconds
    \return number of milliseconds left if interrupted, else 0.
    \bug interruptions not implemented.
*/
extern inline unsigned int msleep(unsigned int msec) {
  (void) wait_event(&tm_sleep_wakeup,sys_time+MSECS_TO_TICKS(msec));
  return 0;
}

//
// kill a process
// FIXME: this belongs in a different header
//
extern void kill(pid_t pid);

#endif
